@@ -49,7 +49,7 @@
 #include <stdbool.h>
 
 
-typedef struct { uint8_t bgr[3]; } bgr_t;
+typedef struct { uint8_t bgr[3]; } mrcal_bgr_t;
 
 #define MRCAL_IMAGE_DECLARE(T, Tname)                                   \
 typedef struct                                                          \
@@ -87,15 +87,32 @@ mrcal_image_ ## Tname ## _crop(mrcal_image_ ## Tname ## _t* image,      \
                                           .h      = h,                  \
                                           .stride = image->stride,      \
                                           .data   = mrcal_image_ ## Tname ## _at(image,x0,y0) }; \
-}                                                                       \
-                                                                        \
+}
+
+#define MRCAL_IMAGE_SAVE_LOAD_DECLARE(T, Tname)                         \
 bool mrcal_image_ ## Tname ## _save (const char* filename, const mrcal_image_ ## Tname ## _t*  image); \
 bool mrcal_image_ ## Tname ## _load( mrcal_image_ ## Tname ## _t*  image, const char* filename);
 
 
-MRCAL_IMAGE_DECLARE(uint8_t,  uint8);
-MRCAL_IMAGE_DECLARE(uint16_t, uint16);
-MRCAL_IMAGE_DECLARE(bgr_t,    bgr);
+// Common images types
+MRCAL_IMAGE_DECLARE(uint8_t,     uint8);
+MRCAL_IMAGE_DECLARE(uint16_t,    uint16);
+MRCAL_IMAGE_DECLARE(mrcal_bgr_t, bgr);
+MRCAL_IMAGE_SAVE_LOAD_DECLARE(uint8_t,     uint8);
+MRCAL_IMAGE_SAVE_LOAD_DECLARE(uint16_t,    uint16);
+MRCAL_IMAGE_SAVE_LOAD_DECLARE(mrcal_bgr_t, bgr);
+
+// Uncommon types. Not everything supports these
+MRCAL_IMAGE_DECLARE(int8_t,  int8);
+MRCAL_IMAGE_DECLARE(int16_t, int16);
+
+MRCAL_IMAGE_DECLARE(int32_t,  int32);
+MRCAL_IMAGE_DECLARE(uint32_t, uint32);
+MRCAL_IMAGE_DECLARE(int64_t,  int64);
+MRCAL_IMAGE_DECLARE(uint64_t, uint64);
+
+MRCAL_IMAGE_DECLARE(float,    float);
+MRCAL_IMAGE_DECLARE(double,   double);
 
 // Load the image into whatever type is stored on disk
 bool mrcal_image_anytype_load(// output
