@@ -418,11 +418,6 @@ A sample valid .cameramodel file:
             for l in note.splitlines():
                 f.write('# ' + l + '\n')
 
-        _validateIntrinsics(self._imagersize,
-                            self._intrinsics)
-        _validateValidIntrinsicsRegion(self._valid_intrinsics_region)
-        _validateExtrinsics(self._extrinsics)
-
         # I write this out manually instead of using repr for the whole thing
         # because I want to preserve key ordering
         f.write("{\n")
@@ -475,7 +470,10 @@ A sample valid .cameramodel file:
     def _read_into_self(self, f):
         r'''Reads in a model from an open file, or the model given as a string
 
-        Note that the string is NOT a filename, it's the model data'''
+        Note that the string is NOT a filename, it's the model data. This
+        function reads the native .cameramodel format only
+
+        '''
 
         # workaround for python3 idiocy
         try:
@@ -731,43 +729,335 @@ ARGUMENTS
                     self._icam_intrinsics = None
                 return
 
-            if type(file_or_model) is str:
 
-                # Some readable file. Read it!
-                def tryread(f, what):
-                    modelstring = f.read()
+
+
+            def parse_as_opencv_or_ros(modelstring):
+                r'''Try to parse model as an opencv/ros string
+
+Supports yaml, json. Supports opencv and ros formats. And the output of
+"rostopic echo" for "sensor_msgs/CameraInfo" messages. This functions tries to
+be general, and accept everything.
+
+This is documented here: https://wiki.ros.org/camera_calibration_parsers
+
+A sample file:
+
+  image_width: 2448
+  image_height: 2050
+  camera_name: prosilica
+  camera_matrix:
+    rows: 3
+    cols: 3
+    data: [4827.94, 0, 1223.5, 0, 4835.62, 1024.5, 0, 0, 1]
+  distortion_model: plumb_bob
+  distortion_coefficients:
+    rows: 1
+    cols: 5
+    data: [-0.41527, 0.31874, -0.00197, 0.00071, 0]
+  rectification_matrix:
+    rows: 3
+    cols: 3
+    data: [1, 0, 0, 0, 1, 0, 0, 0, 1]
+  projection_matrix:
+    rows: 3
+    cols: 4
+    data: [4827.94, 0, 1223.5, 0, 0, 4835.62, 1024.5, 0, 0, 0, 1, 0]
+
+A sample sensor_msgs/CameraInfo message:
+
+  $ rostopic echo -n1 -b tst.bag /camera/camera_info
+
+  ....
+  height: 600
+  width: 960
+  distortion_model: "rational_polynomial"
+  D: [1.5, 0.4, 0.1, -9.2e-05, 0.1, 1.9, 0.9, 0.2]
+  K: [420.1, 0.1, 479.1, 0.1, 420.1, 295.1, 0.1, 0.1, 1.1]
+  R: [0.9998926520347595, 0.014629560522735119, -0.0007753203972242773, -0.014624223113059998, 0.9998719692230225, 0.006493249908089638, 0.0008702144841663539, -0.006481214426457882, 0.9999786019325256]
+  P: [600.0, 0.0, 480.0, -20.3, 0.0, 600.0, 300.0, 0.0, 0.0, 0.0, 1.0, 0.0]
+  ....
+
+These are apparently trying to include rectification in the model, which is
+silly: I should be able to have a single camera with extrinsics. And there're no
+clear extrinsics stored here either, and I must figure out what is really
+intended here.
+
+Here the only "extrinsics" relate the camera to its rectified version.
+Rectification may rotate a camera, but may NOT translate it.
+
+- From previous experience, the rotation in R is R_leftrect_cam
+
+- P[:,3] are scaled translations: t*fx as described here:
+
+    https://docs.opencv.org/2.4/modules/calib3d/doc/camera_calibration_and_3d_reconstruction.html#stereorectify
+
+- From previous experience, the translation in P[:,3]/fx is
+  t_rightrect_leftrect.
+
+So for the purposes of extrinsics, the reference is the "left-rectified" camera
+
+                '''
+
+                def load():
                     try:
-                        self._read_into_self(modelstring)
-                        return
-                    except CameramodelParseException:
+                        import yaml
+                        return yaml.safe_load(modelstring)
+                    except Exception as e:
+                        e1 = e
                         pass
 
-                    # Couldn't read the file as a .cameramodel. Does a .cahvor
-                    # work?
-
-                    # This is more complicated than it looks. I want to read the
-                    # .cahvor file into self, but the current cahvor interface
-                    # wants to generate a new model object. So I do that, write
-                    # it as a .cameramodel-formatted string, and then read that
-                    # back into self. Inefficient, but this is far from a hot
-                    # path
-                    from . import cahvor
                     try:
-                        model = cahvor.read_from_string(modelstring)
+                        import json
+                        return json.loads(modelstring)
+                    except Exception as e:
+                        e2 = e
+                        pass
+
+                    raise CameramodelParseException(f"Couldn't parse yaml (Exception '{e1}') or json (Exception '{e2}')")
+
+
+                # output
+                model = dict()
+
+                # This will raise the exception, as needed
+                model_in = load()
+
+                def find_array( key_sequences,
+                                dtype, shape,
+                                model):
+                    r'''Search a model dictionary for a given array
+
+There are multiple ros/opencv data formats that aren't identical, so this
+general function is used to find the data. The args are:
+
+
+- key_sequences: a list of key sequences to look for. A key sequence such as
+  ('camera_matrix','data') will find
+  model[...][...][...].....['camera_matrix']['data']. At most one such matching
+  sequence is allowed. If we find more than one, this function will throw an
+  error
+
+- dtype: the requested dtype of the array we look for. If the data can't be
+  interpreted in this way, I barf
+
+- shape: the shape of array we look for. This function will try to reshape into
+  this shape, and accept any array that may be reshaped in this way. Pass () to
+  interpret this value as a scalar. Pass None to not reshape, and pass the resulting array as is
+
+- model: the input dict
+
+                    '''
+
+
+                    def find_sequence(s,d, at=''):
+                        for k in d.keys():
+                            if isinstance(k,str) and k == s[0]:
+                                if len(s) == 1:
+                                    if isinstance(d[k],dict):
+                                        return None,None
+                                    return d[k], f"{at}['{k}']"
+                                if not isinstance(d[k], dict):
+                                    return None,None
+                                return \
+                                    find_sequence(s[1:], d[k], at=f"{at}['{k}']")
+                        return None,None
+
+                    matches = [find_sequence(s,model) for s in key_sequences]
+
+                    matches = [(m,at) for m,at in matches if m is not None]
+                    if len(matches) == 0:
+                        raise CameramodelParseException(f"None of required key sequences '{key_sequences}' found. Must have exactly one")
+                    if len(matches) > 1:
+                        raise CameramodelParseException(f"More than one of required key sequences '{key_sequences}' found. Must have exactly one")
+
+                    m,at = matches[0]
+                    try:
+                        m = np.array(m, dtype=dtype)
                     except:
-                        raise Exception(f"Couldn't parse {what} as a camera model (.cameramodel or .cahvor)") from None
+                        raise CameramodelParseException(f"Could not parse model{at} as dtype = {dtype}")
+
+                    if shape is not None:
+                        try:
+                            m = m.reshape(shape)
+                        except:
+                            raise CameramodelParseException(f"Could not parse model{at} with shape {shape}. Input has shape {m.shape}")
+
+                        if len(shape) == 0: m = m.item() # extract the value if we have a scalar
+
+                    return m,at
+
+
+                M,M_at = \
+                    find_array( ( ('camera_matrix','data'),
+                                  ('camera_matrix',),
+                                  ('K',) ),
+                                dtype = float,
+                                shape = (3,3),
+                                model = model_in )
+                if M[0,1] != 0 or \
+                   M[1,0] != 0 or \
+                   M[2,0] != 0 or \
+                   M[2,1] != 0 or \
+                   M[2,2] != 1:
+                    raise CameramodelParseException(f"model {M_at} should have [fx 0 cx; 0 fy cy; 0 0 1] structure")
+
+                P,P_at = \
+                    find_array( ( ('projection_matrix','data'),
+                                  ('projection_matrix',),
+                                  ('P',) ),
+                                dtype = float,
+                                shape = (3,4),
+                                model = model_in )
+                if P[1,3] != 0 or \
+                   P[2,3] != 0:
+                    raise CameramodelParseException(f"model {P_at} expected to have last column of [x*fx,0,0], but instead have: {P[:,3]}")
+
+                try:
+                    R,R_at = \
+                        find_array( ( ('rotation','data'),
+                                      ('rotation',),
+                                      ('R',) ),
+                                    dtype = float,
+                                    shape = (3,3),
+                                    model = model_in )
+                except:
+                    R = mrcal.identity_R()
+                    R_at = 'default'
+
+                # Special-case P=0 or R=0. Sometimes I see this. Set everything to identity
+                if nps.norm2(P.ravel()) == 0:
+                    P[:,:3] = np.eye(3)
+                if nps.norm2(R.ravel()) == 0:
+                    R       = np.eye(3)
+
+                lensmodel,lensmodel_at = \
+                    find_array( ( ('distortion_model',),),
+                                dtype = str,
+                                shape = (),
+                                model = model_in )
+
+                map_lensmodel = \
+                    dict(plumb_bob           = 'LENSMODEL_OPENCV5',
+                         rational_polynomial = 'LENSMODEL_OPENCV8')
+
+                try:
+                    model['lensmodel'] = map_lensmodel[lensmodel]
+                except:
+                    if lensmodel == 'equidistant':
+                        raise CameramodelParseException('"equidistant" OpenCV model not supported yet')
+                    else:
+                        raise CameramodelParseException(f"Unknown OpenCV model \"{lensmodel}\". I only about: {list(map_lensmodel.keys())}")
+
+                distortion,distortion_at = \
+                    find_array( ( ('distortion_coefficients','data'),
+                                  ('distortion_coefficients',),
+                                  ('D',)),
+                                dtype = float,
+                                shape = None, # Any shape. Do not reshape
+                                model = model_in )
+
+                try:
+                    model['intrinsics'] = \
+                          [M[0,0],M[1,1],M[0,2],M[1,2]] + \
+                          list(distortion)
+                except Exception as e:
+                    raise CameramodelParseException(f"No model {distortion_at}")
+                # not checking len(distortion_coefficients);
+                #_read_into_self() will do that
+
+                image_width,image_width_at = \
+                    find_array( ( ('image_width',),
+                                  ('width',)),
+                                dtype = int,
+                                shape = (),
+                                model = model_in )
+                image_height,image_height_at = \
+                    find_array( ( ('image_height',),
+                                  ('height',)),
+                                dtype = int,
+                                shape = (),
+                                model = model_in )
+
+                model['imagersize'] = [image_width,image_height]
+
+                Rt_ref_cam = np.zeros((4,3), dtype=float)
+                Rt_ref_cam[:3,:] = R
+                if nps.norm2((nps.matmult(R, R.T) - np.eye(3)).ravel()) > 1e-12:
+                    raise CameramodelParseException(f"R must be a valid rotation. Instead it is {R}")
+
+
+                # In rectified coords ("ref" coords here) I want the camera to
+                # sit at -P[:,3] / P[0,0]
+                Rt_ref_cam[ 3,:] = -P[:,3] / P[0,0]
+
+                # extrinsics are rt_fromref
+                model['extrinsics'] = list(mrcal.rt_from_Rt(mrcal.invert_Rt(Rt_ref_cam)))
+
+                return repr(model)
+
+
+            # Some readable file. Read it!
+            def tryread(f, what):
+                r'''Try all the formats I support'''
+
+                modelstring = f.read()
+
+                errors = dict()
+                try:
+                    self._read_into_self(modelstring)
+                    return
+                except CameramodelParseException as e:
+                    errors['cameramodel'] = e
+                    pass
+
+                try:
+                    self._read_into_self(parse_as_opencv_or_ros(modelstring))
+                    return
+                except CameramodelParseException as e:
+                    errors['yaml_or_json'] = e
+                    pass
+
+
+                # Couldn't read the file in any known method. Last try: does a
+                # .cahvor work?
+
+                # This is more complicated than it looks. I want to read the
+                # .cahvor file into self, but the current cahvor interface
+                # wants to generate a new model object. So I do that, write
+                # it as a .cameramodel-formatted string, and then read that
+                # back into self. Inefficient, but this is far from a hot
+                # path
+                try:
+                    from . import cahvor
+                    model = cahvor.read_from_string(modelstring)
                     modelfile = io.StringIO()
                     model.write(modelfile)
                     self._read_into_self(modelfile.getvalue())
+                    return
+                except Exception as e:
+                    errors['cahvor'] = e
+
+                raise Exception(f"Couldn't parse {what}. Errors for each attempt: {errors}")
+
+            if isinstance(file_or_model, str):
 
                 if file_or_model == '-':
+                    if sys.stdin.isatty():
+                        # This isn't an error per-se. But most likely the user
+                        # ran something like "mrcal-to-cahvor" without
+                        # redirecting any data into it. Without this check the
+                        # program will sit there, waiting for input. Which will
+                        # look strange to an unsuspecting user
+                        raise Exception("Trying to read a model from standard input, but no file is being redirected into it")
                     tryread(sys.stdin, "STDIN")
                 else:
                     with open(file_or_model, 'r') as openedfile:
                         tryread(openedfile, f"file '{file_or_model}'")
-                return
+            else:
+                # I assume this is a readable file
+                tryread(file_or_model, "file object")
 
-            self._read_into_self(file_or_model)
             return
 
 
@@ -852,7 +1142,11 @@ ARGUMENTS
             ')'
 
 
-    def write(self, f, *, note=None, cahvor=False):
+    def write(self, f,
+              *,
+              note    = None,
+              cahvor  = False,
+              _opencv = False):
         r'''Write out this camera model to disk
 
 SYNOPSIS
@@ -878,24 +1172,135 @@ ARGUMENTS
 RETURNED VALUES
 
 None
+
         '''
 
-        if cahvor:
+        # opencv models may be written by setting _opencv=True. But this has no
+        # clear way to specify extrinsics, and requires specifying stereo
+        # rectification, so this is undocumented for now
+        known_format_options = ('cahvor','_opencv')
+
+        NformatOptions = 0
+        for o in known_format_options:
+            if locals()[o]:
+                NformatOptions += 1
+        if NformatOptions > 1:
+            raise Exception(f"At most 1 of {known_format_options} may be given.")
+
+
+        _validateIntrinsics(self._imagersize,
+                            self._intrinsics)
+        _validateValidIntrinsicsRegion(self._valid_intrinsics_region)
+        _validateExtrinsics(self._extrinsics)
+
+
+
+        def write_cahvor(f):
             from . import cahvor
             cahvor.write(f, self, note)
-            return
 
-        if type(f) is str:
-            if re.match(".*\.cahv(or(e)?)?$", f):
-                from . import cahvor
-                cahvor.write(f, self, note)
+        def write_opencv(f):
+            r'''Write out an opencv-format file
 
+This has no clear way to specify extrinsics, and requires specifying stereo
+rectification, so this is undone and undocumented for now. This function always
+sets an identity extrinsics transform and completely made-up projection and
+rectification matrices
+
+
+This is documented here: https://wiki.ros.org/camera_calibration_parsers
+
+A sample file:
+
+  image_width: 2448
+  image_height: 2050
+  camera_name: prosilica
+  camera_matrix:
+    rows: 3
+    cols: 3
+    data: [4827.94, 0, 1223.5, 0, 4835.62, 1024.5, 0, 0, 1]
+  distortion_model: plumb_bob
+  distortion_coefficients:
+    rows: 1
+    cols: 5
+    data: [-0.41527, 0.31874, -0.00197, 0.00071, 0]
+  rectification_matrix:
+    rows: 3
+    cols: 3
+    data: [1, 0, 0, 0, 1, 0, 0, 0, 1]
+  projection_matrix:
+    rows: 3
+    cols: 4
+    data: [4827.94, 0, 1223.5, 0, 0, 4835.62, 1024.5, 0, 0, 0, 1, 0]
+
+            '''
+
+            if   self._intrinsics[0] == 'LENSMODEL_OPENCV5':
+                distortion_model = 'plumb_bob'
+                distortions      = self._intrinsics[1][4:]
+            elif self._intrinsics[0] == 'LENSMODEL_OPENCV4':
+                distortion_model = 'plumb_bob'
+                distortions      = nps.glue(self._intrinsics[1][4:], 0,
+                                            axis = -1)
+            if   self._intrinsics[0] == 'LENSMODEL_PINHOLE':
+                distortion_model = 'plumb_bob'
+                distortions      = nps.glue(self._intrinsics[1][4:],
+                                            np.zeros((5,), dtype=float),
+                                            axis = -1)
+            elif self._intrinsics[0] == 'LENSMODEL_OPENCV8':
+                distortion_model = 'rational_polynomial'
+                distortions      = self._intrinsics[1][4:]
+            # I don't support "equidistant" yet
             else:
-                with open(f, 'w') as openedfile:
-                    self._write( openedfile, note )
+                raise Exception(f"OpenCV yaml can't store the \"{self._intrinsics[0]}\" model")
+            fxycxy = self._intrinsics[1][:4]
 
+
+            M = np.array(((fxycxy[0], 0,         fxycxy[2]),
+                          (        0, fxycxy[1], fxycxy[3]),
+                          (        0,         0,         1)),
+                         dtype=float)
+
+            f.write(f"image_width: {self._imagersize[0]}\nimage_height: {self._imagersize[1]}\ncamera_name: mrcalmodel\n")
+
+            f.write(f"camera_matrix:\n  rows: 3\n  cols: 3\n  data: [{fxycxy[0]}, 0, {fxycxy[2]}, 0, {fxycxy[1]}, {fxycxy[3]}, 0, 0, 1]\n")
+            f.write(f"distortion_model: {distortion_model}\n")
+            f.write(f"distortion_coefficients:\n  rows: 1\n  cols: {len(distortions)}\n  data: [")
+            np.savetxt(f, nps.atleast_dims(distortions,-2),
+                       delimiter=', ',
+                       newline  ='',
+                       fmt='%.12g')
+            f.write("]\n")
+            f.write('''rectification_matrix:
+  rows: 3
+  cols: 3
+  data: [1, 0, 0, 0, 1, 0, 0, 0, 1]
+projection_matrix:
+  rows: 3
+  cols: 4
+  data: [1, 0, 0, 0,  0, 1, 0, 0,  0, 0, 1, 0]
+''')
+
+
+        write_function = None
+        if   cahvor:  write_function = write_cahvor
+        elif _opencv: write_function = write_opencv
+
+        if isinstance(f, str):
+            with open(f, 'w') as openedfile:
+                if write_function is not None:
+                    write_function(openedfile)
+                elif re.match(r".*\.cahv(or(e)?)?$", f):
+                    write_cahvor(openedfile)
+                elif re.match(r".*\.ya?ml$", f):
+                    write_opencv(openedfile)
+                else:
+                    self._write( openedfile, note )
         else:
-            self._write( f, note )
+            if write_function is not None:
+                write_function(f)
+            else:
+                self._write( f, note )
 
 
     def intrinsics(self,
