@@ -57,6 +57,10 @@ We return a transformation that minimizes the sum 2-norm of the misalignment:
 
 We return an Rt transformation to map points in set 1 to points in set 0.
 
+At least 3 sets of points must be passed-in to produce a non-singular result. If
+a set of inputs produces a singular result, a (4,3) array of 0 is returned. This
+is not a valid Rt transform, and is used to signal an error.
+
 A similar computation can be performed to instead align a set of UNIT VECTORS to
 compute an optimal rotation matrix R by calling align_procrustes_vectors_R01().
 
@@ -77,6 +81,9 @@ The Rt transformation in an array of shape (4,3). We return the optimal
 transformation to align the given point clouds. The transformation maps points
 TO coord system 0 FROM coord system 1.
 
+If the inputs were singular or insufficient, a (4,3) array of 0 is returned.
+This is not a valid Rt transform, and is used to signal an error.
+
     """
     if weights is None:
         weights = np.ones(p0.shape[:-1], dtype=float)
@@ -96,6 +103,12 @@ def _align_procrustes_points_Rt01(p0, p1, weights):
                       nps.transpose(p1 - np.mean(p1, axis=-1)[..., np.newaxis]))
     V,S,Ut = np.linalg.svd(Mt)
 
+    # I look at the second-lowest singular value. One 0 singular value is OK;
+    # two isn't
+    if S[-2] < 1e-12:
+        # Poorly-defined problem. Return error
+        return np.zeros((4,3), dtype=float)
+
     R = nps.matmult(V, Ut)
 
     # det(R) is now +1 or -1. If it's -1, then this contains a mirror, and thus
@@ -148,6 +161,10 @@ misalignment:
 
 We return a rotation to map vectors in set 1 to vectors in set 0.
 
+At least 2 sets of vectors must be passed-in to produce a non-singular result.
+If a set of inputs produces a singular result, a (3,3) array of 0 is returned.
+This is not a valid rotation, and is used to signal an error.
+
 A similar computation can be performed to instead align a set of POINTS to
 compute an optimal transformation Rt by calling align_procrustes_points_Rt01().
 
@@ -168,6 +185,9 @@ The rotation matrix in an array of shape (3,3). We return the optimal rotation
 to align the given vector sets. The rotation maps vectors TO coord system 0 FROM
 coord system 1.
 
+If the inputs were singular or insufficient, a (3,3) array of 0 is returned.
+This is not a valid rotation, and is used to signal an error.
+
     """
 
     if weights is None:
@@ -187,6 +207,12 @@ def _align_procrustes_vectors_R01(v0, v1, weights):
     Mt = nps.matmult( v0*weights, nps.transpose(v1) )
     V,S,Ut = np.linalg.svd(Mt)
 
+    # I look at the second-lowest singular value. One 0 singular value is OK;
+    # two isn't
+    if S[-2] < 1e-12:
+        # Poorly-defined problem. Return error
+        return np.zeros((3,3), dtype=float)
+
     R = nps.matmult(V, Ut)
 
     # det(R) is now +1 or -1. If it's -1, then this contains a mirror, and thus
@@ -634,7 +660,7 @@ element (difference consists of N separate polygons)
 
     diff = Polygon(positive).difference(Polygon(negative))
     if isinstance(diff, (MultiPolygon,GeometryCollection)):
-        diff = list(diff)
+        diff = list(diff.geoms)
     elif isinstance(diff, Polygon):
         diff = [diff]
     else:
@@ -1160,9 +1186,39 @@ None
             calobject_warp.ravel()[:] = b[ivar0:ivar0+Nvars_calobject_warp]
 
 
-def _sorted_eig(C):
-    'like eig(), but the results are sorted by eigenvalue'
-    l,v = np.linalg.eig(C)
+def sorted_eig(M):
+    r'''Compute eigenvalues, eigenvectors; sorted results returned
+
+SYNOPSIS
+
+    # p is an array of shape (N,3): a point cloud. I find a normal to a fitted
+    # plane
+    p -= np.mean(p, axis=-2)
+
+    l,v = mrcal.sorted_eig(nps.matmult(nps.transpose(p),p))
+
+    n = v[:,0]
+
+    # n is the eigenvector corresponding to the smallest eigenvalue
+
+This function calls numpy.linalg.eig(), and sorts the output by eigenvalue, in
+ascending order
+
+ARGUMENTS
+
+- M: the matrix to eigendecompose
+
+RETURNED VALUE
+
+Same as that of numpy.linalg.eig(). A tuple:
+
+- Eigenvalues, sorted in ascending order
+
+- Eigenvectors, sorted by the corresponding eigenvalue. Each eigenvector is
+  stored in a COLUMN of this array
+
+    '''
+    l,v = np.linalg.eig(M)
     i = np.argsort(l)
     return l[i], v[:,i]
 
@@ -1197,23 +1253,22 @@ def _plot_args_points_and_covariance_ellipse(q, what):
              ( q, dict(_with = 'dots',
                          tuplesize = -2)) )
 
+def residuals_board(optimization_inputs,
+                    *,
+                    icam_intrinsics     = None,
+                    residuals           = None,
+                    return_observations = False,
 
-def residuals_chessboard(optimization_inputs,
-                         *,
-                         icam_intrinsics     = None,
-                         residuals           = None,
-                         return_observations = False,
-
-                         # for backwards-compatibility only; same as
-                         # icam_intrinsics
-                         i_cam               = None):
+                    # for backwards-compatibility only; same as
+                    # icam_intrinsics
+                    i_cam               = None):
     r'''Compute and return the chessboard residuals
 
 SYNOPSIS
 
     model = mrcal.cameramodel(model_filename)
 
-    gp.plot( mrcal.residuals_chessboard(
+    gp.plot( mrcal.residuals_board(
                optimization_inputs = model.optimization_inputs(),
                icam_intrinsics     = icam_intrinsics ).ravel(),
              histogram = True,
@@ -1326,6 +1381,8 @@ else:
             residuals_board   [idx, ...    ], \
             observations_board[idx, ..., :2]
 
+# For backwards compatibility
+residuals_chessboard = residuals_board
 
 def residuals_point(optimization_inputs,
                     *,
@@ -1446,3 +1503,213 @@ else:
         return \
             residuals_point   [idx, ...    ], \
             observations_point[idx, ..., :2]
+
+def R_aligned_to_vector(v):
+    r'''Compute a rotation to map a given vector to [0,0,1]
+
+SYNOPSIS
+
+    # I have a plane that passes through a point p, and has a normal n. I
+    # compute a transformation from the world to a coord system aligned to the
+    # plane, with p at the origin. R_plane_world p + t_plane_world = 0:
+
+    Rt_plane_world = np.zeros((4,3), dtype=float)
+    Rt_plane_world[:3,:] = mrcal.R_aligned_to_vector(n)
+    Rt_plane_world[ 3,:] = -mrcal.rotate_point_R(Rt_plane_world[:3,:],p)
+
+This rotation is not unique: adding any rotation around v still maps v to
+[0,0,1]. An arbitrary acceptable rotation is returned.
+
+ARGUMENTS
+
+- v: a numpy array of shape (3,). The vector that the computed rotation maps to
+  [0,0,1]. Does not need to be normalized. Must be non-0
+
+RETURNED VALUES
+
+The rotation in a (3,3) array
+
+    '''
+    z = v/nps.mag(v)
+    if np.abs(z[0]) < .9:
+        x = np.array((1,0,0.))
+    else:
+        x = np.array((0,1,0.))
+    x -= nps.inner(x,z)*z
+    x /= nps.mag(x)
+    y = np.cross(z,x)
+    return nps.cat(x,y,z)
+
+def write_point_cloud_as_ply(filename,
+                             points,
+                             *,
+                             color     = None,
+                             binary    = True):
+
+    r'''Write a point cloud to a .ply file
+
+SYNOPSIS
+
+    print( points.shape )
+    ===>
+    (1000, 3)
+
+    write_point_cloud_as_ply( "points.ply", points )
+
+The point cloud to visualize may contain colors for each point: either as a BGR
+triplet or as a single intensity value. The colors may be given either in the
+trailing columns of the "points" array or in a separate "color" array.
+
+Both binary and ascii .ply formats are supported, with the binary format the
+default.
+
+ARGUMENTS
+
+- filename: the filename of the .ply file being written
+
+- points: the numpy array of points being visualized. Required. One of 3 shapes
+  is allowed:
+
+  - shape (N,3) if no color information is given here; may be given in the
+    separate "color" array
+  - shape (N,4) if a grayscale intensity is given in the last column
+  - shape (N,6) if a BGR color is given in the last 3 columns
+
+  If color is given here, the separate "color" array must be None
+
+- color: the numpy array of colors corresponding to each point in "points".
+  Optional. If the colors are given here, they must NOT be given in "points":
+  points.shape MUST be (N,3). If not None, 2 shapes are allowed:
+
+  - shape (N,)  if we have a grayscale intensity
+  - shape (N,3) if we have BGR colors
+
+- binary: optional boolean, defaulting to True. Selects the binary or ascii .ply
+  format.
+
+
+RETURNED VALUE
+
+None
+'''
+
+
+    if points.ndim != 2:
+        raise Exception(f"points.shape must be (N,3) or (N,4) or (N,6), but points.ndim={points.ndim}")
+    N,Ntrailing = points.shape
+
+    if not (Ntrailing == 3 or \
+            Ntrailing == 4 or \
+            Ntrailing == 6):
+        raise Exception(f"points.shape[-1] must be one of (3,4,6) to indicate no-color or grayscale or bgr colors respectively. Instead got points.shape[-1] = {points.shape[-1]}")
+
+    if color is not None:
+
+        if Ntrailing != 3:
+            raise Exception("Both 'points' and 'color' are specifying color information. At most one of those should provide it")
+
+        if color.ndim > 2:
+            raise Exception(f"color.shape must be (N,) or (N,3), but color.ndim={color.ndim}")
+        if N != color.shape[0]:
+            raise Exception(f"Inconsistent point counts: len(points)={len(points)} but len(color)={len(color)}")
+
+        if not (color.shape == (N,) or \
+                color.shape == (N,3) ):
+            raise Exception(f"color.shape must be (N,) or (N,3), but color.shape={color.shape}")
+
+    else:
+        # No "color" specified
+        if Ntrailing == 4:
+            # But it exists as grayscale in the points. Separate it into the
+            # "color" array
+            color  = points[:,3]
+            points = points[:,:3]
+        elif Ntrailing == 6:
+            # But it exists as bgr in the points. Separate it into the
+            # "color" array
+            color  = points[:,3:]
+            points = points[:,:3]
+
+
+
+    # I now have points.shape = (N,3) and color is None or (N,) or (N,3)
+
+
+
+    if binary:
+        ply_type = 'binary_little_endian'
+    else:
+        ply_type = 'ascii'
+
+    if color is not None and color.ndim == 1:
+        # grayscale
+        dtype = np.dtype([ ('xyz',np.float32,3), ('i', np.uint8) ])
+        header_color = '''property uchar intensity
+'''
+    elif color is not None and color.ndim == 2:
+        # bgr
+        dtype = np.dtype([ ('xyz',np.float32,3), ('rgb', np.uint8, 3) ])
+        header_color = '''property uchar red
+property uchar green
+property uchar blue
+'''
+    else:
+        dtype = np.dtype([ ('xyz',np.float32,3), ])
+        header_color = ''
+
+    ply_header = f'''ply
+format {ply_type} 1.0
+element vertex {N}
+property float x
+property float y
+property float z
+{header_color}end_header
+'''.encode()
+
+
+    with open(filename, 'wb') as f:
+        f.write(ply_header)
+
+        if binary:
+            binary_ply = np.empty( (N,),
+                                   dtype = dtype)
+            binary_ply['xyz'] = points.astype(np.float32)
+            if color is not None and color.ndim == 1:
+                # grayscale
+                binary_ply['i'  ] = color .astype(np.uint8)
+            elif color is not None and color.ndim == 2:
+                # bgr
+                binary_ply['rgb'][:,0] = color[:,2]
+                binary_ply['rgb'][:,1] = color[:,1]
+                binary_ply['rgb'][:,2] = color[:,0]
+
+            binary_ply.tofile(f)
+        else:
+            if color is not None and color.ndim == 1:
+                # grayscale
+                pbgr = nps.glue(points,
+                                nps.dummy(color, -1),
+                                axis = -1)
+                np.savetxt(f,
+                           pbgr,
+                           fmt      = ('%.1f','%.1f','%.1f','%d'),
+                           comments = '',
+                           header   = '')
+            elif color is not None and color.ndim == 2:
+                # bgr
+                pbgr = nps.glue(points,
+                                color[:,(2,)],
+                                color[:,(1,)],
+                                color[:,(0,)],
+                                255*np.ones((N,1)),
+                                axis = -1)
+                np.savetxt(f,
+                           pbgr,
+                           fmt      = ('%.1f','%.1f','%.1f','%d','%d','%d','%d'),
+                           comments = '',
+                           header   = '')
+            else:
+                np.savetxt(f, points,
+                           fmt      = ('%.1f','%.1f','%.1f',),
+                           comments = '',
+                           header   = '')
