@@ -30,7 +30,8 @@
 // and on any theory of liability, whether in contract, strict liability,
 // or tort (including negligence or otherwise) arising in any way out of
 
-#define _GNU_SOURCE
+// Apparently I need this in MSVC to get constants
+#define _USE_MATH_DEFINES
 
 #include <math.h>
 #include <float.h>
@@ -84,8 +85,8 @@ void mrcal_R_from_r_full( // outputs
 
     double theta = sqrt(norm2r);
 
-    double c,s;
-    sincos(theta, &s, &c);
+    double s = sin(theta);
+    double c = cos(theta);
     double c1 = 1. - c;
     double itheta = 1./theta;
 
