@@ -6,7 +6,8 @@
 //
 //     http://www.apache.org/licenses/LICENSE-2.0
 
-#define _GNU_SOURCE
+// Apparently I need this in MSVC to get constants
+#define _USE_MATH_DEFINES
 
 #include <stdio.h>
 #include <string.h>
